import { Subscription, PartialObserver } from '../types';
export declare class Cell<T> {
    static of<T>(initial: T): Cell<T>;
    get current(): T;
    [Symbol.toStringTag]: string;
    map<R>(fn: (value: T) => R): Cell<R>;
    /**
     * @deprecated Should be only used for interop.
     */
    subscribe(observer?: PartialObserver<T>): Subscription;
    /**
     * @deprecated Should be only used for interop.
     */
    subscribe(next?: (value: T) => void, error?: (error: any) => void, complete?: () => void): Subscription;
}
export declare type Track = {
    <C>(cell: Cell<C>): C;
    error(cell: Cell<any>): any;
};
export declare class Reaction {
    constructor(callback: () => void);
    /**
     *
     * @param program
     * @returns return value of `program`
     * @throws User-land errors in program()
     */
    track<T>(program: ($: Track) => T): T;
    dispose(): void;
}
export declare class IncrementalReaction {
    constructor(callback: () => void);
    track<T>(program: ($: Track) => T): T;
    continue<T>(program: ($: Track) => T): T;
    private getHistoryReads;
    dispose(): void;
}
export declare function mutable<T>(initialValue: T, options?: {
    comparator: (a: T, b: T) => boolean;
}): [
    Cell<T>,
    (value: (T extends Function ? never : T) | ((current: T) => T)) => void
];
export declare function computed<T>(expr: ($: Track) => T, initial?: T, options?: {
    comparator: (a: T, b: T) => boolean;
}): Cell<T>;
declare type Unwrap<T> = {
    [Key in keyof T]: T[Key] extends Cell<infer CellType> ? CellType : T[Key];
};
export declare function combined<TArray extends any[]>(array: TArray): Cell<Unwrap<TArray>>;
export declare function combined<TObject extends object>(object: TObject): Cell<Unwrap<TObject>>;
export {};
